#!/usr/bin/env python3
# NOTE: part 2 of example caproto_to_ophyd.py
'''
Running this script assumes you have that IOC running initially.

This script will connect to that server through ophyd, initialize pydm, and
spawn an autogenerated typhos screen for the device.
'''
import sys

try:
    import pydm
    import typhos
except ImportError as ex:
    print(f'pydm/typhos unavailable: {ex}')
    pydm = None

from caproto.ioc_examples.too_clever.caproto_to_ophyd import GroupDevice


def run_typhos(prefix='integration:'):
    # See Steps 1...3 in caproto_to_ophyd.py
    dev = GroupDevice(prefix=prefix, name='dev')
    print(dev, dev.read_attrs)
    dev.wait_for_connection()
    get_value = dev.get()

    print('Current values are:')
    print(get_value)

    # Step 4 make a GUI in a line or two
    if pydm is not None:
        app = pydm.PyDMApplication()
        suite = typhos.TyphosSuite.from_device(dev, pin=True)
        # typhos_display.method_panel.add_method(dev.get_random.call)
        suite.show()
        sys.exit(app.exec_())


if __name__ == '__main__':
    try:
        prefix = sys.argv[1]
    except IndexError:
        prefix = 'integration:'
    run_typhos(prefix=prefix)
